@@ -649,65 +649,6 @@ class UrlParseTestCase(unittest.TestCase):
             self.assertEqual(p.scheme, "http")
             self.assertEqual(p.geturl(), "http://www.python.org/javascript:alert('msg')/?query=something#fragment")
 
-    def test_urlsplit_strip_url(self):
-        noise = bytes(range(0, 0x20 + 1))
-        base_url = "http://User:Pass@www.python.org:080/doc/?query=yes#frag"
-
-        url = noise.decode("utf-8") + base_url
-        p = urllib.parse.urlsplit(url)
-        self.assertEqual(p.scheme, "http")
-        self.assertEqual(p.netloc, "User:Pass@www.python.org:080")
-        self.assertEqual(p.path, "/doc/")
-        self.assertEqual(p.query, "query=yes")
-        self.assertEqual(p.fragment, "frag")
-        self.assertEqual(p.username, "User")
-        self.assertEqual(p.password, "Pass")
-        self.assertEqual(p.hostname, "www.python.org")
-        self.assertEqual(p.port, 80)
-        self.assertEqual(p.geturl(), base_url)
-
-        url = noise + base_url.encode("utf-8")
-        p = urllib.parse.urlsplit(url)
-        self.assertEqual(p.scheme, b"http")
-        self.assertEqual(p.netloc, b"User:Pass@www.python.org:080")
-        self.assertEqual(p.path, b"/doc/")
-        self.assertEqual(p.query, b"query=yes")
-        self.assertEqual(p.fragment, b"frag")
-        self.assertEqual(p.username, b"User")
-        self.assertEqual(p.password, b"Pass")
-        self.assertEqual(p.hostname, b"www.python.org")
-        self.assertEqual(p.port, 80)
-        self.assertEqual(p.geturl(), base_url.encode("utf-8"))
-
-        # Test that trailing space is preserved as some applications rely on
-        # this within query strings.
-        query_spaces_url = "https://www.python.org:88/doc/?query=    "
-        p = urllib.parse.urlsplit(noise.decode("utf-8") + query_spaces_url)
-        self.assertEqual(p.scheme, "https")
-        self.assertEqual(p.netloc, "www.python.org:88")
-        self.assertEqual(p.path, "/doc/")
-        self.assertEqual(p.query, "query=    ")
-        self.assertEqual(p.port, 88)
-        self.assertEqual(p.geturl(), query_spaces_url)
-
-        p = urllib.parse.urlsplit("www.pypi.org ")
-        # That "hostname" gets considered a "path" due to the
-        # trailing space and our existing logic...  YUCK...
-        # and re-assembles via geturl aka unurlsplit into the original.
-        # django.core.validators.URLValidator (at least through v3.2) relies on
-        # this, for better or worse, to catch it in a ValidationError via its
-        # regular expressions.
-        # Here we test the basic round trip concept of such a trailing space.
-        self.assertEqual(urllib.parse.urlunsplit(p), "www.pypi.org ")
-
-        # with scheme as cache-key
-        url = "//www.python.org/"
-        scheme = noise.decode("utf-8") + "https" + noise.decode("utf-8")
-        for _ in range(2):
-            p = urllib.parse.urlsplit(url, scheme=scheme)
-            self.assertEqual(p.scheme, "https")
-            self.assertEqual(p.geturl(), "https://www.python.org/")
-
     def test_attributes_bad_port(self):
         """Check handling of invalid ports."""
         for bytes in (False, True):
@@ -715,7 +656,7 @@ class UrlParseTestCase(unittest.TestCase):
                 for port in ("foo", "1.5", "-1", "0x10", "-0", "1_1", " 1", "1 ", "६"):
                     with self.subTest(bytes=bytes, parse=parse, port=port):
                         netloc = "www.example.net:" + port
-                        url = "http://" + netloc + "/"
+                        url = "http://" + netloc
                         if bytes:
                             if netloc.isascii() and port.isascii():
                                 netloc = netloc.encode("ascii")
@@ -727,6 +668,24 @@ class UrlParseTestCase(unittest.TestCase):
                         with self.assertRaises(ValueError):
                             p.port
 
+    def test_attributes_bad_scheme(self):
+        """Check handling of invalid schemes."""
+        for bytes in (False, True):
+            for parse in (urllib.parse.urlsplit, urllib.parse.urlparse):
+                for scheme in (".", "+", "-", "0", "http&", "६http"):
+                    with self.subTest(bytes=bytes, parse=parse, scheme=scheme):
+                        url = scheme + "://www.example.net"
+                        if bytes:
+                            if url.isascii():
+                                url = url.encode("ascii")
+                            else:
+                                continue
+                        p = parse(url)
+                        if bytes:
+                            self.assertEqual(p.scheme, b"")
+                        else:
+                            self.assertEqual(p.scheme, "")
+
     def test_attributes_without_netloc(self):
         # This example is straight from RFC 3261.  It looks like it
         # should allow the username, hostname, and port to be filled
@@ -1106,16 +1065,24 @@ class UrlParseTestCase(unittest.TestCase):
         self.assertEqual(p1.params, 'phone-context=+1-914-555')
 
     def test_Quoter_repr(self):
-        quoter = urllib.parse.Quoter(urllib.parse._ALWAYS_SAFE)
+        quoter = urllib.parse._Quoter(urllib.parse._ALWAYS_SAFE)
         self.assertIn('Quoter', repr(quoter))
 
+    def test_clear_cache_for_code_coverage(self):
+        urllib.parse.clear_cache()
+
+    def test_urllib_parse_getattr_failure(self):
+        """Test that urllib.parse.__getattr__() fails correctly."""
+        with self.assertRaises(AttributeError):
+            unused = urllib.parse.this_does_not_exist
+
     def test_all(self):
         expected = []
         undocumented = {
             'splitattr', 'splithost', 'splitnport', 'splitpasswd',
             'splitport', 'splitquery', 'splittag', 'splittype', 'splituser',
             'splitvalue',
-            'Quoter', 'ResultBase', 'clear_cache', 'to_bytes', 'unwrap',
+            'ResultBase', 'clear_cache', 'to_bytes', 'unwrap',
         }
         for name in dir(urllib.parse):
             if name.startswith('_') or name in undocumented:
@@ -1133,7 +1100,8 @@ class UrlParseTestCase(unittest.TestCase):
         hex_chars = {'{:04X}'.format(ord(c)) for c in illegal_chars}
         denorm_chars = [
             c for c in map(chr, range(128, sys.maxunicode))
-            if (hex_chars & set(unicodedata.decomposition(c).split()))
+            if unicodedata.decomposition(c)
+            and (hex_chars & set(unicodedata.decomposition(c).split()))
             and c not in illegal_chars
         ]
         # Sanity check that we found at least one such character
@@ -1308,6 +1276,12 @@ class Utility_Tests(unittest.TestCase):
 
 class DeprecationTest(unittest.TestCase):
 
+    def test_Quoter_deprecation(self):
+        with self.assertWarns(DeprecationWarning) as cm:
+            old_class = urllib.parse.Quoter
+            self.assertIs(old_class, urllib.parse._Quoter)
+        self.assertIn('Quoter will be removed', str(cm.warning))
+
     def test_splittype_deprecation(self):
         with self.assertWarns(DeprecationWarning) as cm:
             urllib.parse.splittype('')
